﻿using System;
using System.Data.SQLite;

namespace EvidenciaUctov.DB
{
	// *************************************************************************
	// *** Context *************************************************************
	// *************************************************************************
	public abstract class Context : object, IDisposable
	{
		// ---------------------------------------------------------------------
		protected SQLiteTransaction transaction;
		private SQLiteCommand command;

		// ---------------------------------------------------------------------
		// --- Context Vlastnosti ----------------------------------------------
		// ---------------------------------------------------------------------
		internal virtual SQLiteTransaction Transaction
		{
			get
			{
				return transaction;
			} // get

			set
			{
				transaction = value;
				if (command != null)
					command.Transaction = Transaction;
			} // set
		}

		// ---------------------------------------------------------------------
		public SQLiteCommand Command
		{
			get
			{
				if (command == null)
				{
					command = new SQLiteCommand()
					{
						Connection = Connection,
						Transaction = Transaction
					};
				} // if

				return command;
			} // get
		}

		// ---------------------------------------------------------------------
		// --- Context Abstraktne Vlastnosti -----------------------------------
		// ---------------------------------------------------------------------
		public abstract SQLiteConnection Connection { get; }

		// ---------------------------------------------------------------------
		// --- Context Metody --------------------------------------------------
		// ---------------------------------------------------------------------
		public void BeginTransaction()
		{
			if (Transaction == null)
				Transaction = Connection.BeginTransaction();
		}

		// ---------------------------------------------------------------------
		public void Commit()
		{
			if (transaction != null)
			{
				transaction.Commit();
				Transaction = null;
			} // if
		}

		// ---------------------------------------------------------------------
		public void Rollback()
		{
			if (transaction != null)
			{
				transaction.Rollback();
				Transaction = null;
			} // if
		}

		// ---------------------------------------------------------------------
		#region IDisposable Members

		// ---------------------------------------------------------------------
		public virtual void Dispose()
		{
			Rollback();
		}

		#endregion
	} // public abstract class Context

	// *************************************************************************
	// *** BaseContext *********************************************************
	// *************************************************************************
	public class BaseContext : Context
	{
		// ---------------------------------------------------------------------
		private SQLiteConnection connection;

		// ---------------------------------------------------------------------
		// --- BaseContext Konstruktor -----------------------------------------
		// ---------------------------------------------------------------------
		internal BaseContext(string connectionString)
		{
			connection = new SQLiteConnection(connectionString);
			connection.Open();
		}

		// ---------------------------------------------------------------------
		// --- BaseContext Vlastnosti ------------------------------------------
		// ---------------------------------------------------------------------
		public override SQLiteConnection Connection
		{
			get { return connection; }
		}

		// ---------------------------------------------------------------------
		// --- BaseContext Metody ----------------------------------------------
		// ---------------------------------------------------------------------
		#region IDisposable Members

		// ---------------------------------------------------------------------
		public override void Dispose()
		{
			base.Dispose();
			connection.Close();
		}

		#endregion
	} // public class BaseContext

	// *************************************************************************
	// *** NestedContext *******************************************************
	// *************************************************************************
	public class NestedContext : Context
	{
		// ---------------------------------------------------------------------
		private Context parentContext;

		// ---------------------------------------------------------------------
		// --- NestedContext Konstruktor ---------------------------------------
		// ---------------------------------------------------------------------
		internal NestedContext(Context parentContext)
		{
			this.parentContext = parentContext;
		}

		// ---------------------------------------------------------------------
		// --- NestedContext Vlastnosti ----------------------------------------
		// ---------------------------------------------------------------------
		public override SQLiteConnection Connection
		{
			get { return parentContext.Connection; }
		}

		// ---------------------------------------------------------------------
		internal override SQLiteTransaction Transaction
		{
			get { return transaction ?? parentContext.Transaction; }
		}
	} // public class NestedContext

	// *************************************************************************
	// *** ContextFactory ******************************************************
	// *************************************************************************
	public static class ContextFactory
	{
		// ---------------------------------------------------------------------
		public static string ConnectionString(string databaza, string heslo)
		{
			string heslovaCast = string.IsNullOrEmpty(heslo) ? string.Empty : ";Password=" + heslo;
			return "Data Source=" + databaza + ";Version=3;Compress=True" + heslovaCast;
		}

		// ---------------------------------------------------------------------
		public static Context Create(string databaza, string heslo)
		{
			return new BaseContext(ConnectionString(databaza, heslo));
		}

		// ---------------------------------------------------------------------
		public static Context Create(Context parentContext)
		{
			return new NestedContext(parentContext);
		}
	} // public static class ContextFactory

} // namespace EvidenciaUctov.DB