﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using EvidenciaUctov.Utils;

namespace EvidenciaUctov.DB
{
	// *************************************************************************
	// *** Layer ***************************************************************
	// *************************************************************************
	public static class Layer
	{
		// ---------------------------------------------------------------------
		internal static string Heslo;
		internal static string Databaza;

		// ---------------------------------------------------------------------
		// --- Layer Enumeracie ------------------------------------------------
		// ---------------------------------------------------------------------
		public enum TypDat
		{
			SerializovanyFilter
		} // TypDat

		// ---------------------------------------------------------------------
		// --- Layer Metody ----------------------------------------------------
		// ---------------------------------------------------------------------
		#region Systemove

		// ---------------------------------------------------------------------
		public static void VytvoritDatabazu(string subor, string heslo)
		{
			if (File.Exists(subor))
				File.Delete(subor);

			try // catch
			{
				// Interne prihlasenie (pouzitie contextu)
				Prihlasit(heslo, subor);

				// Vytvorime databazu a dotiahneme jej strukturu na poslednu znamu uroven
				using (Context context = Utils.VytvoritContext(subor, heslo))
					InicializovatDatabazu(context);
			} // try
			catch
			{
				try // catch
				{
					if (File.Exists(subor))
						File.Delete(subor);
				} // try
				catch (Exception exceptionChybaMazania)
				{
					Log.Write(exceptionChybaMazania);
				} // catch

				throw;
			} // catch
		}

		// ---------------------------------------------------------------------
		private static void InicializovatDatabazu(Context parentContext)
		{
			SpustitSkript(/*skript*/1, parentContext);
		}

		// ---------------------------------------------------------------------
		private static void SpustitSkript(int cisloSkriptu, Context parentContext = null)
		{
			try // catch
			{
				// Nacitame skript
				string nazovSkriptu = "DB.Skripty." + cisloSkriptu.ToString("0000") + ".sql";
				var streamSkript = ResourceUtils.GetResource(typeof(Layer), nazovSkriptu);
				if (streamSkript == null)
					throw new Exception("Interná chyba: Skript '" + nazovSkriptu + "' sa nenašiel!");

				string skript;
				using (var reader = new StreamReader(streamSkript))
					skript = reader.ReadToEnd();

				// Spustime skript
				using (var context = Utils.VytvoritContextPrihlaseny(parentContext))
				{
					context.BeginTransaction();

					context.Command.CommandText = skript;	// Telo skriptu
					context.Command.CommandType = CommandType.Text;
					context.Command.CommandTimeout = 0;		// Vypneme time-outy

					// Spustime skript
					context.Command.ExecuteNonQuery();

					// Zvysime cislo verzie - kedze skript zbehoh OK - v pripade chyby sa revertne aj skript
					NastavitHodnotuNastavenia(Nastavenia.KeyDBVerzia, cisloSkriptu, context);	// Musi ist v ramci tranzakcie

					context.Commit();
				} // using
			} // try
			catch (Exception e)
			{
				Log.Write("Upgrade '" + cisloSkriptu.ToString() + "' zlyhal: " + e.Message);
				throw;
			} // catch
		}

		// ---------------------------------------------------------------------
		public static void Prihlasit(int pozadovanaVerziaDB, string subor, string heslo)
		{
			try // catch
			{
				if (!File.Exists(subor))
					throw new Exception("Súbor neexistuje!");

				// Interne prihlasenie
				Prihlasit(heslo, subor);

				if (Nastavenia.DBVerzia == 0)
					throw new Exception("Interná chyba: Problém s databázou!");
				if (Nastavenia.DBVerzia > pozadovanaVerziaDB)
					throw new Exception("Nie je dovolené prihlásiť sa na databázu k novšej verzii programu!");

				// Spustime vsetky potrebne upgradovacie skripty
				int spustitSkript = Nastavenia.DBVerzia;
				if (spustitSkript < pozadovanaVerziaDB)
				{
					do
					{
						SpustitSkript(++spustitSkript);
					} // do
					while (spustitSkript < pozadovanaVerziaDB);

					string text = "Databáza úspešne upgradovaná na verziu: " + pozadovanaVerziaDB.ToString();
					Log.Write(text);
					App.Sprava(text);
				} // if
			} // try
			catch
			{
				Odhlasit();
				throw;
			} // catch
		}

		// ---------------------------------------------------------------------
		private static void Prihlasit(string heslo, string subor)
		{
			Heslo = heslo;
			Databaza = subor;
		}

		// ---------------------------------------------------------------------
		public static void Odhlasit()
		{
			Heslo = null;
		}

		// ---------------------------------------------------------------------
		public static bool JePrihlaseny()
		{
			return (Heslo != null);
		}

		#endregion // Systemove

		// ---------------------------------------------------------------------
		#region Nastavenia

		// ---------------------------------------------------------------------
		public static string DajHodnotuNastavenia(string kluc, Context parentContext = null)
		{
			if (string.IsNullOrEmpty(kluc))
				throw new Exception(Nastavenia.VynimkaPrazdnyKluc);
			kluc = kluc.ToUpper();

			using (var context = Utils.VytvoritContextPrihlaseny("SELECT Hodnota FROM Nastavenia WHERE Kluc = @Kluc", parentContext))
			{
				context.Command.Parameters.AddWithValue("@Kluc", kluc);

				using (var reader = new DataReader(context.Command.ExecuteReader(CommandBehavior.SingleResult)))
				{
					if (!reader.Read())
						return null;

					return reader.ReadString("Hodnota") ?? string.Empty;
				} // using
			} // using
		}

		// ---------------------------------------------------------------------
		public static bool ExistujeNastavenie(string kluc, Context parentContext = null)
		{
			return DajHodnotuNastavenia(kluc, parentContext) != null;
		}

		// ---------------------------------------------------------------------
		public static void VymazatNastavenie(string kluc)
		{
			if (string.IsNullOrEmpty(kluc))
				throw new Exception(Nastavenia.VynimkaPrazdnyKluc);
			kluc = kluc.ToUpper();

			// Pre prazdne hodnoty vymazeme nastavenie
			using (Context context = Utils.VytvoritContextPrihlaseny("DELETE FROM Nastavenia WHERE Kluc = @Kluc"))
			{
				context.Command.Parameters.AddWithValue("@Kluc", kluc);
				
				context.Command.ExecuteNonQuery();
			} // using
		}

		// ---------------------------------------------------------------------
		public static void NastavitHodnotuNastavenia(string kluc, object hodnota, Context parentContext = null)
		{
			if (string.IsNullOrEmpty(kluc))
				throw new Exception();
			kluc = kluc.ToUpper();

			if (hodnota == null)
				VymazatNastavenie(kluc);
			else
			{
				string query = ExistujeNastavenie(kluc, parentContext) ?
					"UPDATE Nastavenia SET Hodnota = @Hodnota WHERE Kluc = @Kluc" :
					"INSERT INTO Nastavenia (Kluc, Hodnota) VALUES (@Kluc, @Hodnota)";

				using (Context context = Utils.VytvoritContextPrihlaseny(query, parentContext))
				{
					context.Command.Parameters.AddWithValue("@Hodnota", hodnota.ToString());
					context.Command.Parameters.AddWithValue("@Kluc", kluc);

					context.Command.ExecuteNonQuery();
				} // using
			} // else
		}

		#endregion // Nastavenia

		// ---------------------------------------------------------------------
		#region Data

		// ---------------------------------------------------------------------
		public static List<SerializovaneData> NacitatData(TypDat typDat)
		{
			using (var context = Utils.VytvoritContextPrihlaseny("SELECT DataID, Nazov, Data FROM Data WHERE DataType = @DataType"))
			{
				context.Command.Parameters.AddWithValue("@DataType", (int)typDat);

				using (var reader = new DataReader(context.Command.ExecuteReader()))
				{
					var result = new List<SerializovaneData>();
					while (reader.Read())
					{
						result.Add(new SerializovaneData()
						{
							DataId = reader.ReadInt32("DataID") ?? 0,
							Nazov = reader.ReadString("Nazov"),
							Data = reader.ReadString("Data")
						});
					} // while

					return result;
				} // using
			} // using
		}

		// ---------------------------------------------------------------------
		public static void VlozitData(TypDat typDat, string nazov, string data)
		{
			if (string.IsNullOrEmpty(data))
				throw new NotSupportedException();

			using (var context = Utils.VytvoritContextPrihlaseny("INSERT INTO Data (DataType, Nazov, Data) VALUES (@DataType, @Nazov, @Data)"))
			{
				context.Command.Parameters.AddWithValue("@DataType", (int)typDat);
				context.Command.Parameters.AddWithValue("@Nazov", nazov);
				context.Command.Parameters.AddWithValue("@Data", data);

				context.Command.ExecuteNonQuery();
			} // using
		}

		// ---------------------------------------------------------------------
		public static void DataVymazat(int dataID)
		{
			using (Context context = Utils.VytvoritContextPrihlaseny("DELETE FROM Data WHERE DataID = @DataID"))
			{
				context.Command.Parameters.AddWithValue("@DataID", dataID);

				context.Command.ExecuteNonQuery();
			} // using
		}

		#endregion // Data

		// ---------------------------------------------------------------------
		#region Ucet

		// ---------------------------------------------------------------------
		public static List<Ucet> NacitatUcty()
		{
			using (var context = Utils.VytvoritContextPrihlaseny("SELECT UcetID, Nazov, BankaID, CisloUctu FROM Ucet"))
			{
				using (var reader = new DataReader(context.Command.ExecuteReader()))
				{
					var ucty = new List<Ucet>();
					while (reader.Read())
					{
						ucty.Add(new Ucet(
							reader.ReadString("Nazov"),
							(Banka)reader.ReadInt32("BankaID"),
							reader.ReadString("CisloUctu")) { UcetID = reader.ReadInt32("UcetID") });
					} // while

					return ucty;
				} // using
			} // using
		}

		// ---------------------------------------------------------------------
		public static void UcetUlozit(Ucet ucet)
		{
			if (ucet == null)
				return;

			string query = ucet.UcetID.HasValue ?
				"UPDATE Ucet SET Nazov = @Nazov, BankaID = @BankaID, CisloUctu = @CisloUctu  WHERE UcetID = @UcetID" :
				"INSERT INTO Ucet (Nazov, BankaID, CisloUctu) VALUES (@Nazov, @BankaID, @CisloUctu)";

			using (var context = Utils.VytvoritContextPrihlaseny(query))
			{
				if (ucet.UcetID.HasValue)
					context.Command.Parameters.AddWithValue("@UcetID", ucet.UcetID.Value);

				context.Command.Parameters.AddWithValue("@Nazov", ucet.Nazov);
				context.Command.Parameters.AddWithValue("@BankaID", (int)ucet.Banka);
				context.Command.Parameters.AddWithValue("@CisloUctu", ucet.CisloUctu);

				context.Command.ExecuteNonQuery();

				// Dohladame zaznam ktory sme volizli
				if (!ucet.UcetID.HasValue)
					ucet.UcetID = Utils.GetLastInsertID("Ucet", context);
			} // using
		}

		#endregion // Ucet

		// ---------------------------------------------------------------------
		#region Obrat

		// ---------------------------------------------------------------------
		public static List<Obrat> NacitatObraty(int ucetID, FilterObdobie filterObdobie = null, Filter filter = null)
		{
			string pridavnyFilter = (filterObdobie == null) ? string.Empty : filterObdobie.ToSql(/*fieldy*/"Mesiac", "Rok");
			string dalsiFilter = (filter == null) ? string.Empty : filter.DajSql();

			using (var context = Utils.VytvoritContextPrihlaseny(
				"SELECT\n" +
				"  *\n" +
				"FROM Obraty\n" +
				"WHERE\n" + 
				"  UcetID = @UcetID" +
				(string.IsNullOrEmpty(pridavnyFilter) ? string.Empty : " AND\n  " + pridavnyFilter) + "\n" +
				(string.IsNullOrEmpty(dalsiFilter) ? string.Empty : " AND\n  (" + dalsiFilter + ")") + "\n" +
				"ORDER BY\n" +
				"  Datum"))
			{
				context.Command.Parameters.AddWithValue("@UcetID", ucetID);

				using (var reader = new DataReader(context.Command.ExecuteReader()))
				{
					var ucty = new List<Obrat>();
					while (reader.Read())
					{
						ucty.Add(new Obrat(
							reader.ReadString("Identifikator"),
							reader.ReadDateTime("Datum").Value,
							reader.ReadString("Poznamka"),
							(DruhObratu)reader.ReadInt32("DruhID"),
							reader.ReadDouble("Ciastka") ?? 0.0,
							PenaznaMena.Euro,
							(reader.ReadInt32("Ignorovat") ?? 0) == 1,
							reader.ReadInt32("ObratID")));
					} // while

					return ucty;
				} // using
			} // using
		}

		// ---------------------------------------------------------------------
		public static int? ObratNajdi(string identifikator)
		{
			if (!string.IsNullOrEmpty(identifikator))
			{
				using (var context = Utils.VytvoritContextPrihlaseny("SELECT ObratID FROM Obrat WHERE Identifikator = @Identifikator"))
				{
					context.Command.Parameters.AddWithValue("@Identifikator", identifikator);

					using (var reader = new DataReader(context.Command.ExecuteReader(CommandBehavior.SingleResult)))
					{
						if (reader.Read())
							return reader.ReadInt32("ObratID");
					} // using
				} // using
			} // if

			return null;
		}

		// ---------------------------------------------------------------------
		public static void ObratUlozit(Obrat obrat, int ucetID)
		{
			int? obratID = ObratNajdi(obrat.Identifikator);	// Existujuce obraty prepiseme
			string query = obratID.HasValue ?
				"UPDATE Obrat SET Identifikator = @Identifikator, Datum = @Datum, Poznamka = @Poznamka, DruhID = @DruhID, Ciastka = @Ciastka, UcetID = @UcetID WHERE ObratID = @ObratID" :
				"INSERT INTO Obrat (Identifikator, Datum, Poznamka, DruhID, Ciastka, UcetID) VALUES (@Identifikator, @Datum, @Poznamka, @DruhID, @Ciastka, @UcetID)";

			using (Context context = Utils.VytvoritContextPrihlaseny(query))
			{
				if (obratID.HasValue)
					context.Command.Parameters.AddWithValue("@ObratID", obratID.Value);

				context.Command.Parameters.AddWithValue("@Identifikator", obrat.Identifikator);
				context.Command.Parameters.AddWithValue("@Datum", obrat.Datum);
				context.Command.Parameters.AddWithValue("@Poznamka", obrat.Poznamka);
				context.Command.Parameters.AddWithValue("@DruhID", (int)obrat.DruhObratu);
				context.Command.Parameters.AddWithValue("@Ciastka", obrat.Ciastka);
				context.Command.Parameters.AddWithValue("@UcetID", ucetID);

				context.Command.ExecuteNonQuery();
			} // using
		}

		// ---------------------------------------------------------------------
		public static void ObratIgnorovat(int obratID, bool ignorovat)
		{
			using (Context context = Utils.VytvoritContextPrihlaseny("UPDATE Obrat SET Ignorovat = @Ignorovat WHERE ObratID = @ObratID"))
			{
				context.Command.Parameters.AddWithValue("@ObratID", obratID);
				context.Command.Parameters.AddWithValue("@Ignorovat", ignorovat ? 1 : 0);

				context.Command.ExecuteNonQuery();
			} // using
		}

		// ---------------------------------------------------------------------
		public static void ObratVymazat(int obratID)
		{
			using (Context context = Utils.VytvoritContextPrihlaseny("DELETE FROM Obrat WHERE ObratID = @ObratID"))
			{
				context.Command.Parameters.AddWithValue("@ObratID", obratID);

				context.Command.ExecuteNonQuery();
			} // using
		}

		#endregion // Obrat

		// ---------------------------------------------------------------------
		#region Statistiky

		// ---------------------------------------------------------------------
		public static List<double> MesacnaStatistikaVydavky(int ucetID, int? rok, bool ignorovane)
		{
			return MesacnaStatistika("JeVydaj = 1", ucetID, rok, ignorovane);
		}

		// ---------------------------------------------------------------------
		public static List<double> MesacnaStatistikaUzivatelska(int ucetID, int? rok, List<MesacnaStatistikaFilterPodmienka> podmienky)
		{
			if (podmienky == null || podmienky.Count == 0)
				return new List<double>();	// Vratime prazdny zoznam

			// Vyskladame filter
			string filter = podmienky.ToSql("Poznamka", "JeVydaj", "DruhObratuID");

			return MesacnaStatistika(filter, ucetID, rok, /*ignorovane*/false);
		}

		// ---------------------------------------------------------------------
		private static List<double> MesacnaStatistika(string pridavnyFilter, int ucetID, int? rok, bool ignorovane)
		{
			// Vyskladame filter
			var podmienky = new List<string>();
			podmienky.Add("UcetID = " + ucetID.ToString());
			
			if (rok.HasValue)
				podmienky.Add("Rok = " + rok.Value.ToString());	
			
			if (!ignorovane)
				podmienky.Add("IFNULL(Ignorovat, 0) <> 1");
			
			podmienky.Add(pridavnyFilter);
			
			podmienky.RemoveAll(item => string.IsNullOrEmpty(pridavnyFilter));
			string filter = string.Join(" AND\n  ", podmienky.ToArray());

			// Nacitame data podla mesiacov
			using (var context = Utils.VytvoritContextPrihlaseny(
				"SELECT Mesiac, SUM(Ciastka) AS Ciastka FROM Obraty WHERE " + filter + " GROUP BY Mesiac ORDER BY Mesiac"))
			{
				using (var reader = new DataReader(context.Command.ExecuteReader()))
				{
					var result = new List<double>();
					
					int mesiacIndex = 1;
					while (reader.Read())
					{
						int? mesiac = reader.ReadInt32("Mesiac");

						if (mesiac.HasValue && mesiac.Value >= 1 && mesiac.Value <= 12)
						{
							double ciastka = reader.ReadDouble("Ciastka", /*default*/0.0).Value;

							// Doplnime chybajuce mesiace
							while (mesiac != mesiacIndex)
							{
								result.Add(0.0);
								mesiacIndex++;
							} // while

							// Pridame mesiac
							result.Add(ciastka);
							mesiacIndex++;
						} // if
					} // while

					// Doplnime chybajuce mesiace
					while (mesiacIndex <= 12)
					{
						result.Add(0.0);
						mesiacIndex++;
					} // while
		
					return result;
				} // using
			} // using
		}

		// ---------------------------------------------------------------------
		private static MesacnaStatistikaFilter FilterNacitat(DataReader reader)
		{
			return new MesacnaStatistikaFilter()
			{
				FilterID = reader.ReadInt32("FilterID"),
				Popis = reader.ReadString("Popis"),
				Ignorovat = (reader.ReadInt32("Ignorovat") ?? 0) == 1,
				SkupinaFiltrovID = reader.ReadInt32("SkupinaFiltrovID")
			};
		}

		// ---------------------------------------------------------------------
		public static MesacnaStatistikaFilter FilterNacitat(int filterID)
		{
			using (var context = Utils.VytvoritContextPrihlaseny("SELECT * FROM Filter WHERE FilterID = @FilterID"))
			{
				context.Command.Parameters.AddWithValue("@FilterID", filterID);

				using (var reader = new DataReader(context.Command.ExecuteReader(CommandBehavior.SingleRow)))
				{
					if (reader.Read())
						return FilterNacitat(reader);

					return null;
				} // using
			} // using
		}

		// ---------------------------------------------------------------------
		public static List<MesacnaStatistikaFilter> NacitatFiltre()
		{
			using (var context = Utils.VytvoritContextPrihlaseny("SELECT * FROM Filter ORDER BY Popis"))
			{
				using (var reader = new DataReader(context.Command.ExecuteReader()))
				{
					var filtre = new List<MesacnaStatistikaFilter>();
					while (reader.Read())
						filtre.Add(FilterNacitat(reader));

					return filtre;
				} // using
			} // using
		}

		// ---------------------------------------------------------------------
		public static void FilterUlozit(MesacnaStatistikaFilter mesacnaStatistikaFilter)
		{
			if (mesacnaStatistikaFilter == null)
				return;

			string query = mesacnaStatistikaFilter.FilterID.HasValue ?
				"UPDATE Filter SET Popis = @Popis, SkupinaFiltrovID = @SkupinaFiltrovID WHERE FilterID = @FilterID" :
				"INSERT INTO Filter (Popis, SkupinaFiltrovID) VALUES (@Popis, @SkupinaFiltrovID)";

			using (Context context = Utils.VytvoritContextPrihlaseny(query))
			{
				if (mesacnaStatistikaFilter.FilterID.HasValue)
					context.Command.Parameters.AddWithValue("@FilterID", mesacnaStatistikaFilter.FilterID);

				context.Command.Parameters.AddWithValue("@Popis", mesacnaStatistikaFilter.Popis);
				context.Command.Parameters.AddWithValue("@SkupinaFiltrovID", mesacnaStatistikaFilter.SkupinaFiltrovID);

				context.Command.ExecuteNonQuery();

				// Dohladame zaznam ktory sme volizli
				if (!mesacnaStatistikaFilter.FilterID.HasValue)
					mesacnaStatistikaFilter.FilterID = Utils.GetLastInsertID("Filter", context);
			} // using
		}

		// ---------------------------------------------------------------------
		public static void FilterIgnorovat(int filterID, bool ignorovat)
		{
			using (Context context = Utils.VytvoritContextPrihlaseny("UPDATE Filter SET Ignorovat = @Ignorovat WHERE FilterID = @FilterID"))
			{
				context.Command.Parameters.AddWithValue("@FilterID", filterID);
				context.Command.Parameters.AddWithValue("@Ignorovat", ignorovat ? 1 : 0);

				context.Command.ExecuteNonQuery();
			} // using
		}

		// ---------------------------------------------------------------------
		public static void FilterVymazat(int filterID)
		{
			using (Context context = Utils.VytvoritContextPrihlaseny("DELETE FROM Filter WHERE FilterID = @FilterID"))
			{
				context.Command.Parameters.AddWithValue("@FilterID", filterID);

				context.Command.ExecuteNonQuery();
			} // using
		}

		// ---------------------------------------------------------------------
		public static List<MesacnaStatistikaFilterPodmienka> NacitatPodmienkyFiltra(int filterID)
		{
			using (var context = Utils.VytvoritContextPrihlaseny("SELECT * FROM FilterPodmienka WHERE FilterID = @FilterID"))
			{
				context.Command.Parameters.AddWithValue("@FilterID", filterID);

				using (var reader = new DataReader(context.Command.ExecuteReader()))
				{
					var filtre = new List<MesacnaStatistikaFilterPodmienka>();
					while (reader.Read())
					{
						var zaznam = new MesacnaStatistikaFilterPodmienka(
							reader.ReadString("Format"),
							reader.ReadInt32("FilterID").Value,
							(DruhObratu)(reader.ReadInt32("DruhObratuID") ?? (int)DruhObratu.Neznamy))
							{
								FilterPodmienkaID = reader.ReadInt32("FilterPodmienkaID")
							};

						filtre.Add(zaznam);
					} // while

					return filtre;
				} // using
			} // using
		}

		// ---------------------------------------------------------------------
		public static void FilterPodmienkaUlozit(MesacnaStatistikaFilterPodmienka data)
		{
			if (data == null)
				return;

			bool novyZaznam = !data.FilterPodmienkaID.HasValue;
			string query = novyZaznam ?
				"INSERT INTO FilterPodmienka (Format, FilterID, DruhObratuID) VALUES (@Format, @FilterID, @DruhObratuID)" :
				"UPDATE FilterPodmienka SET Format = @Format, DruhObratuID = @DruhObratuID WHERE FilterPodmienkaID = @FilterPodmienkaID";

			using (Context context = Utils.VytvoritContextPrihlaseny(query))
			{
				if (!novyZaznam)
					context.Command.Parameters.AddWithValue("@FilterPodmienkaID", data.FilterPodmienkaID);
				else
					context.Command.Parameters.AddWithValue("@FilterID", data.FilterID);

				context.Command.Parameters.AddWithValue("@Format", data.Format);
				context.Command.Parameters.AddWithValue("@DruhObratuID", (int?)data.DruhObratu);

				context.Command.ExecuteNonQuery();

				// Dohladame zaznam ktory sme volizli
				if (novyZaznam)
					data.FilterPodmienkaID = Utils.GetLastInsertID("Filter", context);
			} // using
		}

		// ---------------------------------------------------------------------
		public static void FilterPodmienkaVymazat(int filterPodmienkaID)
		{
			using (Context context = Utils.VytvoritContextPrihlaseny("DELETE FROM FilterPodmienka WHERE FilterPodmienkaID = @FilterPodmienkaID"))
			{
				context.Command.Parameters.AddWithValue("@FilterPodmienkaID", filterPodmienkaID);

				context.Command.ExecuteNonQuery();
			} // using
		}

		// ---------------------------------------------------------------------
		public static List<SkupinaFiltrovMesacnejStatistiky> NacitatSkupinyFiltrov()
		{
			using (var context = Utils.VytvoritContextPrihlaseny("SELECT * FROM SkupinaFiltrov"))
			{
				using (var reader = new DataReader(context.Command.ExecuteReader()))
				{
					var filtre = new List<SkupinaFiltrovMesacnejStatistiky>();
					while (reader.Read())
					{
						filtre.Add(new SkupinaFiltrovMesacnejStatistiky(reader.ReadString("Popis"))
						{
							SkupinaFiltrovID = reader.ReadInt32("SkupinaFiltrovID"),
						});
					} // while

					return filtre;
				} // using
			} // using
		}

		// ---------------------------------------------------------------------
		public static void SkupinyFiltrovUlozit(SkupinaFiltrovMesacnejStatistiky data)
		{
			if (data == null)
				return;

			bool novyZaznam = !data.SkupinaFiltrovID.HasValue;
			string query = novyZaznam ?
				"INSERT INTO SkupinaFiltrov (Popis) VALUES (@Popis)" :
				"UPDATE SkupinaFiltrov SET Popis = @Popis WHERE SkupinaFiltrovID = @SkupinaFiltrovID";

			using (Context context = Utils.VytvoritContextPrihlaseny(query))
			{
				if (!novyZaznam)
					context.Command.Parameters.AddWithValue("@SkupinaFiltrovID", data.SkupinaFiltrovID);

				context.Command.Parameters.AddWithValue("@Popis", data.Popis);

				context.Command.ExecuteNonQuery();

				// Dohladame zaznam ktory sme volizli
				if (novyZaznam)
					data.SkupinaFiltrovID = Utils.GetLastInsertID("SkupinaFiltrov", context);
			} // using
		}

		// ---------------------------------------------------------------------
		public static void SkupinyFiltrovVymazat(int skupinaFiltrovID)
		{
			using (Context context = Utils.VytvoritContextPrihlaseny("DELETE FROM SkupinaFiltrov WHERE SkupinaFiltrovID = @SkupinaFiltrovID"))
			{
				context.Command.Parameters.AddWithValue("@SkupinaFiltrovID", skupinaFiltrovID);

				context.Command.ExecuteNonQuery();
			} // using
		}

		#endregion // Statistiky

		// ---------------------------------------------------------------------
		public static string ToSql(this FilterObdobie filter, string nazovStlpcaMesiac, string nazovStlpcaRok)
		{
			var podmienky = new List<string>();
			if (filter.Rok != null)
				podmienky.Add(string.Format("{0} = {1}", nazovStlpcaRok, filter.Rok.Value.ToString()));
			
			if (filter.Mesiace != null && filter.Mesiace.Length != 0 && filter.Mesiace.Length != 12)
				podmienky.Add(string.Format("{0} IN ({1})", nazovStlpcaMesiac, string.Join(", ", (from mesiac in filter.Mesiace select mesiac.ToString()).ToArray())));

			return string.Join(" AND\n", podmienky.ToArray());
		}

		// ---------------------------------------------------------------------
		private static string ToSql(this MesacnaStatistikaFilterPodmienka podmienka, string nazovStlpcaPopis, string nazovStlpcaJeVydaj, string nazovStlpcaDruhObratu)
		{
			var podmienky = new List<string>();
			podmienky.Add(string.Format("{0} LIKE '%{1}%' COLLATE NOCASE", nazovStlpcaPopis, podmienka.Format));

			if (podmienka.JeVydaj)
				podmienky.Add(string.Format("{0} = 1", nazovStlpcaJeVydaj));

			if (podmienka.DruhObratu != DruhObratu.Neznamy && podmienka.DruhObratu != DruhObratu.Irelevantne)
				podmienky.Add(string.Format("{0} = {1}", nazovStlpcaDruhObratu, (int)podmienka.DruhObratu));

			return "(" + string.Join(" AND\n", podmienky.ToArray()) + ")";
		}
		
		// ---------------------------------------------------------------------
		public static string ToSql(this List<MesacnaStatistikaFilterPodmienka> podmienky, string nazovStlpcaPopis, string nazovStlpcaJeVydaj, string nazovStlpcaDruhObratu)
		{
			var podmienkySql = (from podmienka in podmienky select podmienka.ToSql(nazovStlpcaPopis, nazovStlpcaJeVydaj, nazovStlpcaDruhObratu));
			return "(" + string.Join(" OR ", podmienkySql.ToArray()) + ")";
		}
	} // public static class Layer

} // namespace EvidenciaUctov.DB