﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Xml.Serialization;
using EvidenciaUctov.Utils;

namespace EvidenciaUctov
{
	// *************************************************************************
	// *** FilterOperator ******************************************************
	// *************************************************************************
	public enum OperatorHodnot
	{
		Ziaden,
		[Description("<")]
		Mensi,
		[Description("<=")]
		MensiAleboRovny,
		[Description("=")]
		Rovny,
		[Description(">")]
		VacsiAleboRobny,
		[Description(">=")]
		Vacsi,
		[Description("Obsahuje")]
		Obsahuje
	} // FilterOperator

	// *************************************************************************
	// *** LogickyOperator *****************************************************
	// *************************************************************************
	public enum LogickyOperator
	{
		Ziaden,
		[Description("A súčastne")]
		And,
		[Description("Alebo")]
		Or
	} // LogickyOperator

	// *************************************************************************
	// *** PolozkaFiltra *******************************************************
	// *************************************************************************
	[XmlInclude(typeof(FilterCislo)),
	 XmlInclude(typeof(FilterRetazec))]
	[Serializable]
	public abstract class PolozkaFiltra
	{
		// ---------------------------------------------------------------------
		public string Stlpec;
		public OperatorHodnot Operator;
		public object Hodnota;
		// ---------------------------------------------------------------------
		protected string Text;

		// ---------------------------------------------------------------------
		// --- PolozkaFiltra Metody --------------------------------------------
		// ---------------------------------------------------------------------
		public abstract OperatorHodnot[] DajZoznamPodporovanychOperatorov();

		// ---------------------------------------------------------------------
		public virtual string DajSql()
		{
			if (!JeValidny())
				throw new Exception("Chybný filter!");

			if (Operator == OperatorHodnot.Obsahuje)
				return string.Format("{0} LIKE '%{1}%'", Stlpec, Hodnota.ToString());

			return string.Format("{0} {1} {2}", Stlpec, SqlOperatorHodnot(Operator), DajSqlHodnotu());
		}

		// ---------------------------------------------------------------------
		public virtual bool JeValidny()
		{
			return Hodnota != null && DajZoznamPodporovanychOperatorov().Contains(Operator);
		}

		// ---------------------------------------------------------------------
		public virtual string DajSqlHodnotu()
		{
			return Hodnota.ToString();
		}

		// ---------------------------------------------------------------------
		public static string SqlOperatorHodnot(OperatorHodnot operatorHodnot)
		{
			switch (operatorHodnot)
			{
				case OperatorHodnot.Mensi:
					return "<";

				case OperatorHodnot.MensiAleboRovny:
					return "<=";

				case OperatorHodnot.Rovny:
					return "=";

				case OperatorHodnot.VacsiAleboRobny:
					return ">=";

				case OperatorHodnot.Vacsi:
					return ">";
			} // switch

			throw new NotSupportedException(operatorHodnot.ToString());
		}

		// ---------------------------------------------------------------------
		public override string ToString()
		{
			if (!string.IsNullOrEmpty(Text))
				return Text;

			return base.ToString();
		}
	} // public abstract class PolozkaFiltra

	// *************************************************************************
	// *** FilterCislo *********************************************************
	// *************************************************************************
	[Serializable]
	public class FilterCislo : PolozkaFiltra
	{
		// ---------------------------------------------------------------------
		// --- FilterCislo Konstruktor -----------------------------------------
		// ---------------------------------------------------------------------
		public FilterCislo()
		{
			// Serializacia
		}

		// ---------------------------------------------------------------------
		public FilterCislo(string text, string stlpec)
		{
			Text = text;
			Stlpec = stlpec;
		}

		// ---------------------------------------------------------------------
		// --- FilterCislo Metody ----------------------------------------------
		// ---------------------------------------------------------------------
		public override OperatorHodnot[] DajZoznamPodporovanychOperatorov()
		{
			return EnumUtils.GetEnumList(/*vynechat*/OperatorHodnot.Ziaden, OperatorHodnot.Obsahuje).ToArray();
		}

		// ---------------------------------------------------------------------
		public override bool JeValidny()
		{
			double cislo;
			return base.JeValidny() && double.TryParse(Hodnota.ToString(), out cislo);
		}
	} // public class FilterCislo

	// *************************************************************************
	// *** FilterRetazec *******************************************************
	// *************************************************************************
	[Serializable]
	public class FilterRetazec : PolozkaFiltra
	{
		// ---------------------------------------------------------------------
		// --- FilterRetazec Konstruktor ---------------------------------------
		// ---------------------------------------------------------------------
		public FilterRetazec()
		{
			// Serializacia
		}

		// ---------------------------------------------------------------------
		public FilterRetazec(string text, string stlpec)
		{
			Text = text;
			Stlpec = stlpec;
		}

		// ---------------------------------------------------------------------
		// --- FilterRetazec Metody --------------------------------------------
		// ---------------------------------------------------------------------
		public override OperatorHodnot[] DajZoznamPodporovanychOperatorov()
		{
			return new[] { OperatorHodnot.Obsahuje };
		}

		// ---------------------------------------------------------------------
		public override string DajSqlHodnotu()
		{
			return "'" + Hodnota.ToString() + "'";
		}
	} // public class FilterCislo

	// *************************************************************************
	// *** Filter **************************************************************
	// *************************************************************************
	[Serializable]
	public sealed class Filter
	{
		// ---------------------------------------------------------------------
		public PolozkaFiltra HlavnyFilter;
		public SerializableDictionary<PolozkaFiltra, LogickyOperator> Filtre = new SerializableDictionary<PolozkaFiltra, LogickyOperator>();

		// ---------------------------------------------------------------------
		// --- PolozkaFiltra Konstruktor ---------------------------------------
		// ---------------------------------------------------------------------
		public Filter()
		{
			// Serializacia
		}

		// ---------------------------------------------------------------------
		public Filter(PolozkaFiltra hlavnyFilter)
		{
			HlavnyFilter = hlavnyFilter;
		}

		// ---------------------------------------------------------------------
		// --- PolozkaFiltra Metody --------------------------------------------
		// ---------------------------------------------------------------------
		public string DajSql()
		{
			string filter = HlavnyFilter.DajSql();

			foreach (var dalsiFilter in Filtre.Keys)
				filter = string.Format("{0} {1} ({2})", filter, SqlLogickyOperator(Filtre[dalsiFilter]), dalsiFilter.DajSql());

			return filter;
		}

		// ---------------------------------------------------------------------
		public static string SqlLogickyOperator(LogickyOperator operatorHodnot)
		{
			switch (operatorHodnot)
			{
				case LogickyOperator.And:
					return "AND";

				case LogickyOperator.Or:
					return "OR";
			} // switch

			throw new NotSupportedException(operatorHodnot.ToString());
		}

		// ---------------------------------------------------------------------
		public void Pridat(PolozkaFiltra filter, LogickyOperator logickyOperator)
		{
			Filtre.Add(filter, logickyOperator);
		}
	} // public abstract class Filter

	// *************************************************************************
	// *** FilterObdobie *******************************************************
	// *************************************************************************
	public class FilterObdobie
	{
		// ---------------------------------------------------------------------
		public int? Rok;
		public int[] Mesiace;
	} // public class FilterObdobie

} // namespace EvidenciaUctov