﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using EvidenciaUctov.Utils;

namespace EvidenciaUctov
{
	// *************************************************************************
	// *** DruhObratu **********************************************************
	// *************************************************************************
	public enum DruhObratu
	{
		[Description("Všetky")]
		Neznamy,	// 0
		[Description("Výber")]
		Vyber,		// 1
		[Description("Platba / Prevod")]
		Platba,		// 2
		[Description("TP / Inkaso")]
		TPaInkaso,	// 3
		Poplatok,	// 4
		Vklad,		// 5
		[Description("Úrok")]
		Urok,		// 6
		[Description("Daň")]
		Dan,		// 7
        Irelevantne	// 8
	} // DruhObratu

	// *************************************************************************
	// *** PenaznaMena *********************************************************
	// *************************************************************************
	public enum PenaznaMena
	{
		Neznama,
		Euro,
		Skk	// Byvala slovenska koruna
	} // PenaznaMena

	// *************************************************************************
	// *** Obrat ***************************************************************
	// *************************************************************************
	public struct Obrat
	{
		// ---------------------------------------------------------------------
		internal int? ObratID;
		internal bool Ignorovat;
		public string Identifikator;
		public DateTime Datum;
		public string Poznamka;
		public DruhObratu DruhObratu;
		public double Ciastka;
		public PenaznaMena Mena;

		// ---------------------------------------------------------------------
		// --- Obrat Konstruktor -----------------------------------------------
		// ---------------------------------------------------------------------
		public Obrat(string identifikator, DateTime datum, string poznamka, DruhObratu druhObratu, double ciastka, PenaznaMena mena, bool ignorovat = false, int? obratID = null)
		{
			Identifikator = identifikator;
			Datum = datum;
			Poznamka = poznamka;
			DruhObratu = druhObratu;
			Ciastka = PohybySubor.DajCiastkuVEurach(ciastka, mena);
			Mena = mena;
			Ignorovat = ignorovat;
			ObratID = obratID;
		}

		// ---------------------------------------------------------------------
		// --- Obrat Metody ----------------------------------------------------
		// ---------------------------------------------------------------------
		public override string ToString()
		{
			string flagIgnorovat = Ignorovat ? string.Empty : "*";
			return
				flagIgnorovat.PadRight(1) + " " +
				Datum.ToShortDateString().PadLeft(13) + " " +
				Ciastka.ToString("C2").PadLeft(12) + " " +
				DruhObratu.Description().PadRight(16) + " " +
				Poznamka;
		}
	} // public class Obrat

	// *************************************************************************
	// *** PohybySubor *********************************************************
	// *************************************************************************
	public abstract class PohybySubor
	{
		// ---------------------------------------------------------------------
		private const double KonverznyKurzEuroSkk = 30.126;

		// ---------------------------------------------------------------------
		// --- PohybySubor Metody ----------------------------------------------
		// ---------------------------------------------------------------------
		public static T OverHodnotu<T>(string hodnota, string menoStlpca)
		{
			object vyslednaHodnota;
			if (typeof(T) == typeof(double))
				vyslednaHodnota = hodnota.ToDouble();
			else if (typeof(T) == typeof(DateTime))
				vyslednaHodnota = hodnota.ToDateTime();
			else
				throw new NotSupportedException();

			if (vyslednaHodnota == null)
				throw new Exception("Nepodarilo sa načítať hodnotu stĺpca '" + menoStlpca + "': '" + hodnota + "'");

			return (T)vyslednaHodnota;
		}

		// ---------------------------------------------------------------------
		public static double DajCiastkuVEurach(double ciastka, PenaznaMena mena)
		{
			switch (mena)
			{
				case PenaznaMena.Euro:
					return ciastka;

				case PenaznaMena.Skk:
					return Math.Round((double)(ciastka / KonverznyKurzEuroSkk), /*decimals*/2);
			} // switch

			throw new NotSupportedException("Nepodporovaná konverzia " + mena.ToString() + " -> euro!");
		}

		// ---------------------------------------------------------------------
		public virtual Obrat[] Parsuj(string subor, out string[] log)
		{
			if (!File.Exists(subor))
				throw new Exception("Súbor neexistuje!");

			// Nacitame vety (nie prazdne)
			var zaznamy = File.ReadAllLines(subor, Global.DefaultEncoding).Where(item => !string.IsNullOrEmpty(item));

			// Inicializacia
			var obraty = new List<Obrat>();
			var internyLog = new List<string>();

			// Kazdu vetu rozparsujeme
			int riadok = 0;
			foreach (string zaznam in zaznamy)
			{
				riadok++;

				try // catch
				{
					obraty.Add(ParsujVetu(zaznam));
				} // try
				catch (Exception e)
				{
					internyLog.Add("[" + riadok.ToString() + "] " + e.Message);
				} // catch
			} // foreach

			log = internyLog.ToArray();
			return obraty.ToArray();
		}

		// ---------------------------------------------------------------------
		// --- PohybySubor Abstraktne Metody -----------------------------------
		// ---------------------------------------------------------------------
		public abstract Obrat ParsujVetu(string veta);
	} // public abstract class PohybySubor

	// *************************************************************************
	// *** PohybyUtils *********************************************************
	// *************************************************************************
	public static class PohybyUtils
	{
		// ---------------------------------------------------------------------
		// --- PohybySubor Abstraktne Metody -----------------------------------
		// ---------------------------------------------------------------------
		public static PohybySubor VytvoritParserPohybu(this Banka banka)
		{
			switch (banka)
			{
				case Banka.Slsp:
					return new Banky.PohybySuborSlsp();
			} // switch

			throw new NotSupportedException("Interná chyba: Neexistuje parser pre banku: " + banka.ToString());
		}
	} // public static class PohybyUtils

} // namespace EvidenciaUctov